@@ -1,51 +1,24 @@
-# MIT License
-#
-# Copyright (c) 2022-present HyperGH
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
 from __future__ import annotations
 
 import inspect
 import os
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Callable
-from typing import Optional
-from typing import TypeVar
-from typing import Union
+import typing as t
 
 import hikari
 
-from .item import DecoratedItem
-from .item import Item
+from .abc.item import DecoratedItem
+from .abc.item import ViewItem
 
-if TYPE_CHECKING:
-    from .context import Context
+if t.TYPE_CHECKING:
+    from .context import ViewContext
     from .view import View
 
-ViewT = TypeVar("ViewT", bound="View")
+    ViewT = t.TypeVar("ViewT", bound="View")
 
 __all__ = ["Button", "button"]
 
 
-class Button(Item[ViewT]):
+class Button(ViewItem):
     """A view component representing a button.
 
     Parameters
@@ -76,34 +49,30 @@ class Button(Item[ViewT]):
     def __init__(
         self,
         *,
-        style: Union[hikari.ButtonStyle, int] = hikari.ButtonStyle.PRIMARY,
-        label: Optional[str] = None,
+        style: t.Union[hikari.ButtonStyle, int] = hikari.ButtonStyle.PRIMARY,
+        label: t.Optional[str] = None,
         disabled: bool = False,
-        custom_id: Optional[str] = None,
-        url: Optional[str] = None,
-        emoji: Union[hikari.Emoji, str, None] = None,
-        row: Optional[int] = None,
+        custom_id: t.Optional[str] = None,
+        url: t.Optional[str] = None,
+        emoji: t.Union[hikari.Emoji, str, None] = None,
+        row: t.Optional[int] = None,
     ) -> None:
-        super().__init__()
+        super().__init__(custom_id, disabled)
 
-        self._style: Union[hikari.ButtonStyle, int] = style
-        self._label: Optional[str] = label
-        self._disabled: bool = disabled
-        self._emoji: Union[str, hikari.Emoji, None] = emoji
-        self._custom_id: Optional[str] = custom_id
-        self._row: Optional[int] = int(row) if row is not None else None
-        self._url: Optional[str] = url
-
-        self._persistent: bool = True if custom_id else False
+        self._style: t.Union[hikari.ButtonStyle, int] = style
+        self._label: t.Optional[str] = label
+        self._emoji: t.Union[str, hikari.Emoji, None] = emoji
+        self._row: t.Optional[int] = int(row) if row is not None else None
+        self._url: t.Optional[str] = url
 
         if self._emoji is None and self._label is None:
             raise TypeError("Must provide at least one of emoji or label")
 
-        if self._custom_id and self._url:
+        if self._is_persistent and self._url:
             raise TypeError("Cannot provide both url and custom_id")
 
-        if self.url is None and self.custom_id is None:
-            self.custom_id = os.urandom(16).hex()
+        if isinstance(self._emoji, str):
+            self._emoji = hikari.Emoji.parse(self._emoji)
 
         if self.url is not None:
             # Assign underlying style
@@ -114,14 +83,14 @@ class Button(Item[ViewT]):
         return hikari.ComponentType.BUTTON
 
     @property
-    def style(self) -> Union[hikari.ButtonStyle, int]:
+    def style(self) -> t.Union[hikari.ButtonStyle, int]:
         """
         The button's style.
         """
         return self._style
 
     @style.setter
-    def style(self, value: Union[hikari.ButtonStyle, int]) -> None:
+    def style(self, value: t.Union[hikari.ButtonStyle, int]) -> None:
         if not isinstance(value, (hikari.ButtonStyle, int)):
             raise TypeError("Expected type hikari.ButtonStyle or int for property style.")
 
@@ -131,25 +100,25 @@ class Button(Item[ViewT]):
         self._style = value
 
     @property
-    def label(self) -> Optional[str]:
+    def label(self) -> t.Optional[str]:
         """
         The button's label. This is the text visible on the button.
         """
         return self._label
 
     @label.setter
-    def label(self, value: Optional[str]) -> None:
+    def label(self, value: t.Optional[str]) -> None:
         self._label = str(value) if value else None
 
     @property
-    def emoji(self) -> Union[str, hikari.Emoji, None]:
+    def emoji(self) -> t.Union[str, hikari.Emoji, None]:
         """
         The emoji that should be visible on the button.
         """
         return self._emoji
 
     @emoji.setter
-    def emoji(self, value: Union[str, hikari.Emoji, None]) -> None:
+    def emoji(self, value: t.Union[str, hikari.Emoji, None]) -> None:
         if value and isinstance(value, str):
             value = hikari.Emoji.parse(value)
 
@@ -158,7 +127,7 @@ class Button(Item[ViewT]):
         self._emoji = value
 
     @property
-    def url(self) -> Optional[str]:
+    def url(self) -> t.Optional[str]:
         """
         The button's URL. If specified, the button will turn into a link button,
         and the style parameter will be ignored.
@@ -175,16 +144,28 @@ class Button(Item[ViewT]):
 
         self._url = value
 
-    def _build(self, action_row: hikari.api.ActionRowBuilder) -> None:
-        button: Union[
-            hikari.api.InteractiveButtonBuilder[hikari.api.ActionRowBuilder],
-            hikari.api.LinkButtonBuilder[hikari.api.ActionRowBuilder],
+    @classmethod
+    def _from_component(cls, component: hikari.PartialComponent, row: t.Optional[int] = None) -> ViewItem:
+        assert isinstance(component, hikari.ButtonComponent)
+
+        return cls(
+            style=component.style,
+            label=component.label,
+            disabled=component.is_disabled,
+            custom_id=component.custom_id,
+            url=component.url,
+            emoji=component.emoji,
+            row=row,
+        )
+
+    def _build(self, action_row: hikari.api.MessageActionRowBuilder) -> None:
+        button: t.Union[
+            hikari.api.InteractiveButtonBuilder[hikari.api.MessageActionRowBuilder],
+            hikari.api.LinkButtonBuilder[hikari.api.MessageActionRowBuilder],
         ]
-        if self.style is hikari.ButtonStyle.LINK:
-            assert self.url is not None
+        if self.url is not None:
             button = action_row.add_button(hikari.ButtonStyle.LINK, self.url)
         else:
-            assert self.custom_id is not None
             button = action_row.add_button(self.style, self.custom_id)
 
         if self.label:
@@ -192,18 +173,19 @@ class Button(Item[ViewT]):
         if self.emoji:
             button.set_emoji(self.emoji)
         button.set_is_disabled(self.disabled)
+
         button.add_to_container()
 
 
 def button(
     *,
-    label: Optional[str] = None,
-    custom_id: Optional[str] = None,
+    label: t.Optional[str] = None,
+    custom_id: t.Optional[str] = None,
     style: hikari.ButtonStyle = hikari.ButtonStyle.PRIMARY,
-    emoji: Optional[Union[str, hikari.Emoji]] = None,
-    row: Optional[int] = None,
+    emoji: t.Optional[t.Union[str, hikari.Emoji]] = None,
+    row: t.Optional[int] = None,
     disabled: bool = False,
-) -> Callable[[Callable[[ViewT, Button[ViewT], Context], Any]], Button[ViewT]]:
+) -> t.Callable[[t.Callable[[ViewT, Button, ViewContext], t.Any]], Button]:
     """A decorator to transform a coroutine function into a Discord UI Button's callback.
     This must be inside a subclass of View.
 
@@ -228,10 +210,10 @@ def button(
         The decorated callback coroutine function.
     """
 
-    def decorator(func: Callable[..., Any]) -> Any:
+    def decorator(func: t.Callable[..., t.Any]) -> t.Any:
         if not inspect.iscoroutinefunction(func):
             raise TypeError("button must decorate coroutine function.")
-        item: Button[Any] = Button(
+        item = Button(
             label=label,
             custom_id=custom_id,
             style=style,
@@ -244,3 +226,26 @@ def button(
         return DecoratedItem(item, func)
 
     return decorator
+
+
+# MIT License
+#
+# Copyright (c) 2022-present HyperGH
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in all
+# copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
